package com.ashokit.problems;

import java.util.ArrayList;
import java.util.List;

class Solution {
    public List<List<Integer>> pacificAtlantic(int[][] heights) {
        List<List<Integer>> result = new ArrayList<>();
        if (heights == null || heights.length == 0) {
            return result;
        }

        int rows = heights.length;
        int cols = heights[0].length;

        boolean[][] pacific = new boolean[rows][cols];
        boolean[][] atlantic = new boolean[rows][cols];

        // Perform DFS from all Pacific border cells
        for (int i = 0; i < rows; i++) {
            dfs(heights, pacific, i, 0, Integer.MIN_VALUE); // Left border (Pacific)
            dfs(heights, atlantic, i, cols - 1, Integer.MIN_VALUE); // Right border (Atlantic)
        }

        // Perform DFS from all Atlantic border cells
        for (int j = 0; j < cols; j++) {
            dfs(heights, pacific, 0, j, Integer.MIN_VALUE); // Top border (Pacific)
            dfs(heights, atlantic, rows - 1, j, Integer.MIN_VALUE); // Bottom border (Atlantic)
        }

        // Find all cells that can reach both oceans
        for (int i = 0; i < rows; i++) {
            for (int j = 0; j < cols; j++) {
                if (pacific[i][j] && atlantic[i][j]) {
                    List<Integer> cell = new ArrayList<>();
                    cell.add(i);
                    cell.add(j);
                    result.add(cell);
                }
            }
        }

        return result;
    }

    private void dfs(int[][] heights, boolean[][] visited, int i, int j, int prevHeight) {
        // Boundary and validity checks
        if (i < 0 || i >= heights.length || j < 0 || j >= heights[0].length || visited[i][j] || heights[i][j] < prevHeight) {
            return;
        }

        // Mark the cell as visited
        visited[i][j] = true;

        // Perform DFS in all four directions
        dfs(heights, visited, i - 1, j, heights[i][j]); // Up
        dfs(heights, visited, i + 1, j, heights[i][j]); // Down
        dfs(heights, visited, i, j - 1, heights[i][j]); // Left
        dfs(heights, visited, i, j + 1, heights[i][j]); // Right
    }
}
